    /**
  * @author Derek Marley dmarley@kaltire.com
  * @version 1.0
  * Service: SocketToJMSEndpoint
  * Date   : May 17, 2008
  * Purpose: This service is used to accept connections on a port and 
  *          send the stream'd data to a JMS endpoint as a message 
  *  
  * Date     Initials  Description
  * -------- --------  ----------------------------------------------
  * 05.17.08 Dm        Created
  */


package com.progress.codeshare.esbservice.SocketToJMSEndpoint;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
//import java.util.Date;
import java.util.Iterator;
import java.util.Map;
 
import com.sonicsw.xq.*;

public class SocketToJMSEndpoint implements XQServiceEx {

	
	// This is the XQLog (the container's logging mechanism).
    private XQLog m_xqLog = null;

    //This is the the log prefix that helps identify this service during logging
    private String m_logPrefix = "socketToJMS";
    
    private static int socketPort ;	
    private XQInitContext m_initialContext = null;
    private static boolean m_shutdown      = false;
    private static SocketServer myServer   = null;
    
    //These hold version information.
    private static int s_major = 1;
    private static int s_minor = 0;
    private static int s_buildNumber = 0;

    /**
     * Constructor for a SocketToJMSEndpoint
     */
	public SocketToJMSEndpoint () {	
	}
	
	/**
     * Initialize the XQService by processing its initialization parameters.
     *
     * <p> This method implements a required XQService method.
     *
     * @param initialContext The Initial Service Context provides access to:<br>
     * <ul>
     * <li>The configuration parameters for this instance of the SocketToJMSEndpoint.</li>
     * <li>The XQLog for this instance of the SocketToJMSEndpoint.</li>
     * </ul>
     * @exception XQServiceException Used in the event of some error.
     */
    public void init(XQInitContext initialContext) throws XQServiceException {
    	XQParameters params = initialContext.getParameters();
        m_xqLog = initialContext.getLog();
        setLogPrefix(params);
        m_xqLog.logInformation(m_logPrefix + " Initializing ...");
        
        writeStartupMessage(params);
        writeParameters(params);
        //perform initilization work.
        socketPort = params.getIntParameter("initPort", XQConstants.PARAM_STRING);
               
        m_initialContext = initialContext;    
        m_xqLog.logInformation(m_logPrefix +" Initialized ...");
        m_xqLog.logInformation("----- Initialization Parameters -----");
        m_xqLog.logInformation("TCP Socket Port: " + socketPort);
        
    }


    /**
     * Handle the arrival of XQMessages in the INBOX.
     *
     * <p> This method implement a required XQService method.
     *
     * @param ctx The service context.
     * @exception XQServiceException Thrown in the event of a processing error.
     */
    public void service(XQServiceContext ctx) throws XQServiceException {
       /* service method does nothing... */
       /* todo; possibly allow administration messages to control this service */
    }
    
    /**
     * Clean up and get ready to destroy the service.
     *
     * <p> This method implement a required XQService method.
     */
    public void destroy() {
		m_xqLog.logInformation(m_logPrefix);
		m_xqLog.logInformation(m_logPrefix + "Destroyed...");
    }
    
    /**
     * Called by the container on container start.
     *
     * <p> This method implement a required XQServiceEx method.
     */
	public void start() {
		m_xqLog.logInformation(m_logPrefix + "Starting...");
		
	    myServer = new SocketServer(socketPort);
				
		try{
			//starts the server in a thread to allow sonic framework to load correctly
			myServer.startListening();
		}
		catch(Exception e){ m_xqLog.logWarning("Issue during start of Server Thread: " + e.getMessage());}
		m_xqLog.logInformation(m_logPrefix + "Started...");
	}
	
    /**
     * Called by the container on container stop.
     *
     * <p> This method implement a required XQServiceEx method.
     */
	public void stop() {
		m_xqLog.logInformation(m_logPrefix + "Stopping...");
		m_shutdown = true;
		myServer.stopListening();
		m_xqLog.logInformation(m_logPrefix + "Stopped...");
	}
	
	/**
     * Clean up and get ready to destroy the service.
     *
     */
	protected void setLogPrefix(XQParameters params) {
		String serviceName = params.getParameter(XQConstants.PARAM_SERVICE_NAME, XQConstants.PARAM_STRING);
		m_logPrefix = "[ " +  serviceName + " ]";
	}
	
    /**
     * Provide access to the service implemented version.
     *
     */
	protected String getVersion(){
		return s_major + "." + s_minor + ". build " + s_buildNumber;  
	}
	
    /**
     * Writes a standard service startup message to the log.
     */
    protected void writeStartupMessage(XQParameters params) {
        final StringBuffer buffer = new StringBuffer();
        String serviceTypeName = params.getParameter(XQConstants.SERVICE_PARAM_SERVICE_TYPE, XQConstants.PARAM_STRING);
        buffer.append("\n\n");
        buffer.append("\t\t " + serviceTypeName + "\n ");
        buffer.append("\t\t Version ");
        buffer.append(" " +getVersion());
        buffer.append("\n");
        m_xqLog.logInformation(buffer.toString());
	}
    
    /**
     * Writes parameters to log.
     */
    protected void writeParameters(XQParameters params) {

        final Map map = params.getAllInfo();
        final Iterator iter = map.values().iterator();

        while (iter.hasNext()) {
            final XQParameterInfo info = (XQParameterInfo) iter.next();

            if (info.getType() == XQConstants.PARAM_XML) {
                m_xqLog.logInformation( m_logPrefix + "Parameter Name =  " + info.getName());
            } else if (info.getType() == XQConstants.PARAM_STRING) {
            	m_xqLog.logInformation( m_logPrefix + "Parameter Name = " + info.getName());
            }

            if (info.getRef() != null) {
            	m_xqLog.logInformation(m_logPrefix +"Parameter Reference = " + info.getRef());

            	//If this is too verbose
            	///then a simple change from logInformation to logDebug
            	//will ensure file content is not displayed
            	//unless the logging level is set to debug for the ESB Container.
            	m_xqLog.logInformation(m_logPrefix +"----Parameter Value Start--------");
            	m_xqLog.logInformation("\n" +  info.getValue() +"\n");
            	m_xqLog.logInformation(m_logPrefix +"----Parameter Value End--------");
            }else{
            	m_xqLog.logInformation(m_logPrefix +"Parameter Value = " + info.getValue());            	
            }
        }
    }
    
    /**
	 * @param str
	 * @return
	 * @throws Exception
	 */
	protected XQEnvelope createDispatchEnvelope(String str) throws Exception {
		String contentType = null;
		XQEnvelope envelope = null;

		// Get the factories from the initContext
		XQEnvelopeFactory envFactory    = m_initialContext.getEnvelopeFactory();
		XQMessageFactory messageFactory = m_initialContext.getMessageFactory();
		contentType = XQConstants.CONTENT_TYPE_TEXT;

        // use the Exit Endpoint(s) of the Service
		envelope = envFactory.createDefaultEnvelope(); 

		XQMessage message = messageFactory.createMessage();

		message.addPart(message.createPart(str, contentType));
		
		// Set the message in the dispatch envelope
		envelope.setMessage(message);

		return envelope;
	}
	
	//**************************************************************************
	// Inner Class(s) vvv
	//**************************************************************************
	
	private class MessageDispatchThread extends Thread 
	{
		public MessageDispatchThread(String str) 
		{
			try 
			{
				XQDispatch dispatcher = m_initialContext.getDispatcher();

				// Create the dispatch envelope
				XQEnvelope envelope = createDispatchEnvelope(str);
				
				dispatcher.dispatch(envelope, XQQualityofService.AT_LEAST_ONCE);
			
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	class SocketServer extends Thread 
	{
		private int m_port;
		ServerSocket serverSocket = null;
		
		private SocketServer(int inPort) 
		{
		   m_port = inPort;
		} // SocketServer
		
		public void run()
		{
		  //int i = 0;
		  
          try 
           {
  			  serverSocket = new ServerSocket(m_port);
  			  m_xqLog.logInformation("SocketToJMSEndpoint Service is waiting for client(s) on port " + serverSocket.getLocalPort());
  			  serverSocket.setSoTimeout(20000);
  			  
              while (m_shutdown == false)
              {
               //i++; m_xqLog.logInformation ("Waiting for Connection:" + i);
               try 
               {
                  new ThreadServer (serverSocket.accept()); 
               }
               catch (SocketTimeoutException ste)
               {
                 //m_xqLog.logInformation ("Timeout Exception Occurred");
               }
              } // while
           } 
           catch (IOException e) 
           { 
              m_xqLog.logError("[1] Error in SocketServer run thread,\n" + e.getMessage());  
           }
           catch (Exception ee)
           {
        	   m_xqLog.logError("[2] Error in SocketServer run thread,\n" + ee.getMessage());
           }
           /*
           finally
           {
       		  try 
       		  {
       		    serverSocket.close();
       		  }
       		  catch (IOException e) {}
           }
           */
		}
		
		protected void stopListening(){
 		  try 
   		  {
   		    serverSocket.close();
   		    stop();
   		  }
   		  catch (IOException e) {}			
		}
		
		protected void startListening(){
			start();
		}
		
	}//SocketServer
	
	class ThreadServer extends Thread
	{
		protected Socket clientSocket;
		private BufferedReader rd;
		
		private ThreadServer(Socket clientSoc){
			clientSocket = clientSoc;
			start();
		}
		
		/**
		 * Socket thread loops waiting for more incoming streams.  
		 * Timeout on the stream is fixed at 20000 ms.
		 * 
		 */
		public void run() 
		{
			//m_xqLog.logInformation("New Communication Thread Started...");
			//m_xqLog.logInformation("TCP/IP thread connection created - "+ new Date());
						
			try
			{
				rd = new BufferedReader(new InputStreamReader(
						clientSocket.getInputStream()));

				String str = null;
				String fullMsg = "";

				while (str == null) {
					str = rd.readLine();
				}

				while (str != null) {
					fullMsg = fullMsg.concat(str);
					str = rd.readLine();
				}
				
				rd.close();

				if (fullMsg.length() > 0) 
				{
					new MessageDispatchThread(fullMsg).start(); 
				} 
				else 
				{
				   m_xqLog.logInformation("No message body received on socket");
				}
				
				clientSocket.close();
				clientSocket = null;
				
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			//m_xqLog.logInformation("Unloading Thread");
		} // run method
	} // Class ThreadServer
}